#include <stdio.h>
#include <stddef.h> // offsetof()

struct S1 {
	char c;    // 1 byte (and some padding)
	int i[2];  // 2*4 bytes (and more padding)
	double v;  // 8 bytes
};


// Still has padding, but all at the end.
struct S2 {
	int i[2];  // 2*4 bytes
	double v;  // 8 bytes
	char c;    // 1 byte
};

/* __attribute__((packed)) is a gcc extension that says to avoid padding,
 * even if that makes the alignment bad.
 */
struct S3 {
	char c;
	int i[2];
	double v;
} __attribute__((packed));

struct S4 {
	char c;
	int i;
	char d;
};

struct S5 {
	int i;
	char c;
	char d;
};

int main(void) {
	struct S1 obj1;
	struct S2 obj2;
	struct S3 obj3;
	struct S4 obj4;
	struct S5 obj5;

	printf("\tsize\toffset\n");
	printf("S1:\n");
	/* offsetof(T, m) gives the offset (in bytes) of the member m
	 * within the structure type T. Unlike sizeof, it only works with
	 * types, not with object names. */
	printf("\t%zu\t%zu\n", sizeof(obj1.c), offsetof(struct S1, c));
	printf("\t%zu\t%zu\n", sizeof(obj1.i), offsetof(struct S1, i));
	printf("\t%zu\t%zu\n", sizeof(obj1.v), offsetof(struct S1, v));
	printf("total: %zu bytes \n", sizeof(struct S1));

	printf("\nS2:\n");
	printf("\t%zu\t%zu\n", sizeof(obj2.i), offsetof(struct S2, i));
	printf("\t%zu\t%zu\n", sizeof(obj2.v), offsetof(struct S2, v));
	printf("\t%zu\t%zu\n", sizeof(obj2.c), offsetof(struct S2, c));
	printf("total: %zu bytes\n", sizeof(struct S2));

	printf("\nS3:\n");
	printf("\t%zu\t%zu\n", sizeof(obj3.c), offsetof(struct S3, c));
	printf("\t%zu\t%zu\n", sizeof(obj3.i), offsetof(struct S3, i));
	printf("\t%zu\t%zu\n", sizeof(obj3.v), offsetof(struct S3, v));
	printf("total: %zu bytes \n", sizeof(struct S3));

	printf("-----\n");

	printf("\nS4:\n");
	printf("\t%zu\t%zu\n", sizeof(obj4.c), offsetof(struct S4, c));
	printf("\t%zu\t%zu\n", sizeof(obj4.i), offsetof(struct S4, i));
	printf("\t%zu\t%zu\n", sizeof(obj4.d), offsetof(struct S4, d));
	printf("total: %zu bytes \n", sizeof(struct S4));

	printf("\nS5:\n");
	printf("\t%zu\t%zu\n", sizeof(obj5.i), offsetof(struct S5, i));
	printf("\t%zu\t%zu\n", sizeof(obj5.c), offsetof(struct S5, c));
	printf("\t%zu\t%zu\n", sizeof(obj5.d), offsetof(struct S5, d));
	printf("total: %zu bytes \n", sizeof(struct S5));
	
	return 0;
}
